/*
 * Copyright (C) 2000  Florian Sander
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

static struct dcc_table WEBSEEN_LISTEN =
{
  "WEBSEEN_LISTEN",
  DCT_VALIDIDX,
  eof_webseen,
  webseen_accept,
  0,
  timeout_listen_webseen,
  display_webseen_accept,
  0,
  NULL,
  0
};

static struct dcc_table WEBSEEN =
{
  "WEBSEEN",
  DCT_VALIDIDX,
  eof_webseen,
  webseen_activity,
  &webseen_timeout,
  timeout_webseen,
  display_webseen,
  expmem_webseen,
  kill_webseen,
#ifdef OLDBOT
  out_webseen,
#else
  out_webseen,
  outdone_webseen
#endif
};

static void start_listen_webseen(int port)
{
  int i, zz;
  char tmp[50];

  stop_listen_webseen();
  sprintf(tmp, "set my-ip \"%s\";", webseen_ip);
  do_tcl("webseen-hack-start",
      "set my-ip-webseen-backup ${my-ip};"
      "set my-hostname-webseen-backup ${my-hostname};"
      "set my-hostname \"\"");
  do_tcl("webseen-hack-setip", tmp);
  zz = open_listen(&port);
  do_tcl("webseen-hack-end",
      "set my-ip ${my-ip-webseen-backup};"
      "set my-hostname ${my-hostname-webseen-backup}");
  if (zz == (-1)) {
    putlog(LOG_MISC, "*", "ERROR! Cannot open listening socket for webseen!");
    return;
  }
  if ((i = new_dcc(&WEBSEEN_LISTEN, 0)) == -1) {
    putlog(LOG_MISC, "*", "ERROR! Cannot open listening socket for webseen! DCC table is full!");
    return;
  }
  dcc[i].sock = zz;
  dcc[i].addr = (IP) (-559026163);
  dcc[i].port = port;
  strcpy(dcc[i].nick, "webseen");
  strcpy(dcc[i].host, "*");
  dcc[i].timeval = now;
  putlog(LOG_MISC, "*", "Now listening for webseen connections on port %d", port);
}

static void stop_listen_webseen()
{
  int i;

  for (i = 0; i < dcc_total; i++) {
    if (dcc[i].type == &WEBSEEN_LISTEN) {
      putlog(LOG_MISC, "*",
      	     "no longer listening for webseen connections on port %d",
             dcc[i].port);
      killsock(dcc[i].sock);
      lostdcc(i);
    } else if (dcc[i].type == &WEBSEEN) {
      putlog(LOG_MISC, "*", "killing webseen connection from %s", dcc[i].host);
      killsock(dcc[i].sock);
      lostdcc(i);
    }
  }
}

static void webseen_activity(int idx, char *buf, int len)
{
  char *cmd, *path, *imask;
  int lev;

  // debug2("%s: %s", dcc[idx].host, buf);
  imask = nmalloc(strlen(dcc[idx].host) + 11);
  sprintf(imask, "webseen!*@%s", dcc[idx].host);
  if (match_ignore(imask)) {
    debug1("Ignoring webseen access from %s", dcc[idx].host);
    webseen_info_access(idx)->code = 401;
    if (webseen_ignore_msg[0])
      dprintf(idx, "HTTP/1.0 401 Access Forbidden\nContent-Type: text/html\n\n%s", webseen_ignore_msg);
    killsock(dcc[idx].sock);
    lostdcc(idx);
    nfree(imask);
    return;
  }
  nfree(imask);
  if (!strncasecmp(buf, "GET ", 4)) {
    if (!webseen_info_access(idx)->cmd) {
      webseen_info_access(idx)->cmd = nmalloc(strlen(buf) + 1);
      strcpy(webseen_info_access(idx)->cmd, buf);
      webseen_info_access(idx)->code = 200;
    }
  }
  cmd = newsplit(&buf);
  if (!strcmp(cmd, "GET")) {
    if (webseen_flood()) {
      webseen_info_access(idx)->code = 401;
      killsock(dcc[idx].sock);
      lostdcc(idx);
      return;
    }
    lev = logmodes(webseen_loglevel);
    if (lev)
      putlog(lev, "*", "%s: GET %s", dcc[idx].host, buf);
    path = newsplit(&buf);
    if (!strncasecmp(path, "/robots.txt", 11)) {
      dprintf(idx, "HTTP/1.1 404 Not Found\nServer: webseen.mod/%s\nConnection: close", MODULE_VERSION);
      dprintf(idx, "Content-Type: text/html; charset=iso-8859-1\n\n");
      dprintf(idx, "<HTML><HEAD><TITLE>404 Not Found</TITLE></HEAD><BODY><H1>Not Found</H1>");
      dprintf(idx, "The requested document was not found on this server.</BODY></HTML>");
      webseen_info_access(idx)->code = 404;
      return;
    }
    dprintf(idx, "HTTP/1.0 200 OK\nServer: webseen.mod/%s\nContent-Type: text/html\n\n", MODULE_VERSION);
    send_webseen(idx, path);
  } else if (!strcasecmp(cmd, "User-Agent:")) {
    if (webseen_info_access(idx)->browser)
      return;
    webseen_info_access(idx)->browser = nmalloc(strlen(buf) + 1);
    strcpy(webseen_info_access(idx)->browser, buf);
  } else if (!strcasecmp(cmd, "Referer:")) {
    if (webseen_info_access(idx)->referer)
      return;
    webseen_info_access(idx)->referer = nmalloc(strlen(buf) + 1);
    strcpy(webseen_info_access(idx)->referer, buf);
  } else if (!buf[0]) {
    dcc[idx].status = 1;
#ifndef OLDBOT
    /* If there's no data in our socket, we immediately get rid of it.
     */
    if (!sock_has_data(SOCK_DATA_OUTGOING, dcc[idx].sock)) {
      killsock(dcc[idx].sock);
      lostdcc(idx);
    }
#endif
  }
}

#ifndef OLDBOT
static void outdone_webseen(int idx)
{
  if (dcc[idx].status) {
    killsock(dcc[idx].sock);
    lostdcc(idx);
  } else
    dcc[idx].status = 1;
}
#endif

static void display_webseen(int idx, char *buf)
{
  sprintf(buf, "webseen");
}

static void display_webseen_accept(int idx, char *buf)
{
  sprintf(buf, "lstn port");
}

static void timeout_webseen(int idx)
{
  killsock(dcc[idx].sock);
  lostdcc(idx);
}

static void timeout_listen_webseen(int idx)
{
  debug0("timeout webseen listen");
  killsock(dcc[idx].sock);
  lostdcc(idx);
}

static void kill_webseen(int idx, void *x)
{
  register struct webseen_clientinfo *p = (struct webseen_clientinfo *) x;
  char ts[41], test[11];
  time_t tt;
  FILE *f;

  Context;
  tt = now;
  if (!p) {
    putlog(LOG_MISC, "*", "Can't kill clientinfo, no pointer. This should not happen!");
    return;
  }
  ctime(&tt);
  /* 05/Feb/2000:12:08:17 +0100 */
  strftime(test, 19, "%z", localtime(&tt));
  if (test[0] != 'z')
    strftime(ts, 40, "%d/%b/%Y:%H:%M:%S %z", localtime(&tt));
  else
    strftime(ts, 40, "%d/%b/%Y:%H:%M:%S", localtime(&tt));
  if (webseen_log[0]) {
    f = fopen(webseen_log, "a");
    if (f == NULL)
      putlog(LOG_MISC, "*", "ERROR writing webseen log.");
    else {
      if (test[0] != 'z')
        fprintf(f,
    	 "%s - - [%s] \"%s\" %d %d \"%s\" \"%s\"\n", dcc[idx].host, ts,
	 p->cmd ? p->cmd : "", p->code, p->traffic,
	 p->referer ? p->referer : "-", p->browser ?  p->browser : "");
      else
        fprintf(f,
    	 "%s - - [%s %+05d] \"%s\" %d %d \"%s\" \"%s\"\n", dcc[idx].host,
	 ts, offset * (-1) * 100, p->cmd   ? p->cmd : "", p->code ,p->traffic,
	 p->referer ? p->referer : "-", p->browser ?  p->browser : "");
      fclose(f);
    }
  }
  if (p->browser)
    nfree(p->browser);
  if (p->referer)
    nfree(p->referer);
  if (p->cmd)
    nfree(p->cmd);
  nfree(p);
}

static int expmem_webseen(void *x)
{
  register struct webseen_clientinfo *p = (struct webseen_clientinfo *) x;
  int tot = sizeof(struct webseen_clientinfo);

  Context;
  if (!p) {
    putlog(LOG_MISC, "*", "Can't expmem clientinfo, no pointer. This should not happen!");
    return 0;
  }
  if (p->browser)
    tot += strlen(p->browser) + 1;
  if (p->referer)
    tot += strlen(p->referer) + 1;
  if (p->cmd)
    tot += strlen(p->cmd) + 1;
  return tot;
}

static void out_webseen(int idx, char *buf, void *x)
{
  register struct webseen_clientinfo *p = (struct webseen_clientinfo *) x;

  if (!p) {
    putlog(LOG_MISC, "*", "No webseen_clientinfo pointer. This should not happen!");
    return;
  }
  p->traffic += strlen(buf);
  tputs(dcc[idx].sock, buf, strlen(buf));
}

static void webseen_accept(int idx, char *buf, int len)
{
  unsigned long ip;
  unsigned short port;
  int j = 0, sock, i;
  char s[UHOSTLEN];

  Context;
  if (dcc_total + 1 >= max_dcc) {
    j = answer(dcc[idx].sock, s, &ip, &port, 0);
    if (j != -1) {
      dprintf(-j, "Sorry, too many connections already.\r\n");
      killsock(j);
    }
    return;
  }
  sock = answer(dcc[idx].sock, s, &ip, &port, 0);
  if (sock < 0) {
    neterror(s);
    putlog(LOG_MISC, "*", "WebSeen.mod: Error accepting webseen connection: %s", s);
    return;
  }
  if ((i = new_dcc(&WEBSEEN, sizeof(struct webseen_clientinfo))) == (-1)) {
    putlog(LOG_MISC, "*", "Error accepting webseen connection. DCC table is full.");
    killsock(sock);
    return;
  }
  dcc[i].sock = sock;
  dcc[i].addr = ip;
  dcc[i].port = port;
  strcpy(dcc[i].nick, "httpseen");
#ifndef OLDBOT
  sprintf(s, "%s", iptostr(my_htonl(ip)));
#else
  sprintf(s, "%lu.%lu.%lu.%lu", (ip >> 24) & 0xff, (ip >> 16) & 0xff,
  	  (ip >> 8) & 0xff, ip & 0xff); /* dw */
#endif
  strcpy(dcc[i].host, s);
  dcc[i].timeval = now;
  dcc[i].status = 0;
  ((struct webseen_clientinfo *) dcc[i].u.other)->traffic = 0;
  ((struct webseen_clientinfo *) dcc[i].u.other)->code = 200;
  ((struct webseen_clientinfo *) dcc[i].u.other)->browser = NULL;
  ((struct webseen_clientinfo *) dcc[i].u.other)->referer = NULL;
  ((struct webseen_clientinfo *) dcc[i].u.other)->cmd = NULL;
}

static int mwseen_time = 0, mwseen_thr = 0;
static int webseen_flood()
{
  if (!maxwebseen_thr || !maxwebseen_time)
    return 0;
  if ((now - mwseen_time) > maxwebseen_time) {
    mwseen_time = now;
    mwseen_thr = 0;
  }
  mwseen_thr++;
  if (mwseen_thr > maxwebseen_thr)
    return 1;
  return 0;
}

static void eof_webseen(int idx)
{
  debug0("eof webseen accept");
  killsock(dcc[idx].sock);
  lostdcc(idx);
}

static char *csplit(char **rest, char divider)
{
  register char *o, *r;

  if (!rest)
    return *rest = "";
  o = *rest;
  while (*o == divider)
    o++;
  r = o;
  while (*o && (*o != divider))
    o++;
  if (*o)
    *o++ = 0;
  *rest = o;
  return r;
}

/* decode_url():
 * Decodes all special characters(%3F == '?', %21 == '!', etc)
 * and returns the decoded url
 */
static char *decode_url(char *paramurl)
{
  char *p, *buf, *url, c, hex[5];
  long int i;

  Context;
  // free url-buffer (global var)
  if (decoded_url)
    nfree(decoded_url);
  // initialize url-buffer
  decoded_url = nmalloc(1);
  decoded_url[0] = 0;
  // copy parameter into a buffer
  buf = nmalloc(strlen(paramurl) + 1);
  strcpy(buf, paramurl);
  url = buf;
  // now loop to get every encoded char
  while ((p = strchr(url, '%'))) {
    // '%' marks the beginning of an encoded char, so cut the string here.
    p[0] = 0;
    // append the first part of the string to our buffer
    decoded_url = nrealloc(decoded_url, strlen(decoded_url) + strlen(url) + 1);
    strcat(decoded_url, url);
    // set the pointer to the beginning of the next string
    url = p + 1;
    // first check if there are enough chars left to decode
    if (strlen(url+1) >= 2) {
      // the number behind '%' is a hex-number which is the ASCII code of
      // the char, so dump the hex into a string and scan it
      snprintf(hex, 5, "0x%c%c", p[1], p[2]);
      i = strtol(hex, NULL, 0);
      if (!i) {
        i = '?';
        debug0("WebSeen: decode_url(): i is 0");
      }
      c = (char) i;
      // now append the decoded char to the url
      decoded_url = nrealloc(decoded_url, strlen(decoded_url) + 1 + 1);
      sprintf(decoded_url, "%s%c", decoded_url, c);
      // increase the pointer to abandon the encoded char
      url += 2;
    } else {
      // just append the original '%' if there weren't enough chars to decode
      decoded_url = nrealloc(decoded_url, strlen(decoded_url) + 1 + 1);
      strcat(decoded_url, "%");
    }
  }
  // finally append the rest of the param to our buffer. There are no encoded
  // chars left.
  decoded_url = nrealloc(decoded_url, strlen(decoded_url) + strlen(url) + 1);
  strcat(decoded_url, url);
  // free the buffer
  nfree(buf);
  Context;
  return decoded_url;
}

/* init_global_vars()
 * inits the global variables to make sure that no weird
 * bugs appear due to unitialized vars, or vars with content
 * from the last request
 */
static void init_global_vars()
{
  glob_query = nrealloc(glob_query, 1);
  glob_query[0] = 0;
  glob_results = glob_result = NULL;
  sglobseendat = NULL;
  glob_fuzzy = fuzzy_search;
  glob_seenerror = 0;
  glob_searchtime = 0.0;
}

/* send_webseen():
 * take the URL string, split the parameters off,
 * calculate seen-results if necessary, and finally
 * send a template to the client
 */
static void send_webseen(int idx, char *buf)
{
  char *urlbuf, *url = NULL;
  char *param, *varname, *value;
  double presearch, aftersearch;
  struct timeval t;
  
  Context;
  // init all global vars
  init_global_vars();
  // init the global seenlang settings
  setsglobs(NULL, "www-user");
  // cut the path from the URL and store it
  urlbuf = decode_url(csplit(&buf, '?'));
  url = nmalloc(strlen(urlbuf) + 1);
  strcpy(url, urlbuf);
  // now parse the parameters
  param = csplit(&buf, '&');
  while (param[0]) {
    // split the name of the parameter of the string
    varname = csplit(&param, '=');
    // and decode the value (just for the case that there are special chars
    // like '?' which can't be transferred in plain-text)
    value = decode_url(param);
    // now check if it is one of the supported parameters
    if (!strcasecmp(varname, "query")) {
      // query is the most important parameter, it contains the mask which
      // the user is searching for
      glob_query = nrealloc(glob_query, strlen(value) + 1);
      strcpy(glob_query, value);
    } else if (!strcasecmp(varname, "fuzzy")) {
      // fuzzy search can be turned on or off
      if (value[0] == '1')
        glob_fuzzy = 1;
      else if (value[0] == '0')
        glob_fuzzy = 0;
    } else if (!strcasecmp(varname, "lang")) {
      // and of course, we can also choose a different language
      setseenlang(value);
    }
    // don't forget to cut off the next parameter, or we'll be stuck in an
    // infinite loop ^_^
    param = csplit(&buf, '&');
  }
  // now it's time to choose what to do
  if (!strcmp(url, "/")) {
    // user accessed the server root? ok, send the root template...
    template_send(idx, "webseen_root");
  } else if (!strcmp(url, "/seen")) {
    // oh, looks like a seen-query... let's process it.
    gettimeofday(&t, NULL);
    presearch = (float) t.tv_sec + (((float) t.tv_usec) / 1000000);
    glob_results = findseens(glob_query, &glob_seenerror, glob_fuzzy);
    gettimeofday(&t, NULL);
    aftersearch = (float) t.tv_sec + (((float) t.tv_usec) / 1000000);
    glob_searchtime = aftersearch - presearch;
    if (!glob_results) {
      // glob_results is NULL, so either there was really no result, or
      // an error occured
      if (glob_seenerror == WS_NORESULT)
        template_send(idx, "webseen_noresult");
      else
        template_send(idx, "webseen_error");
    } else {
      // ok, we got our results, so lets output them
      template_send(idx, "webseen_results");
    }
  }
  if (glob_results)
    free_seenresults();
  Assert(url);
  nfree(url);
}